package q1k2.pxlx.train.plugin.form;

import com.alibaba.fastjson.JSON;
import com.kingdee.util.StringUtils;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.datamodel.events.PackageDataEvent;
import kd.bos.fileservice.FileItem;
import kd.bos.fileservice.FileService;
import kd.bos.fileservice.FileServiceFactory;
import kd.bos.form.*;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.operatecol.OperationColItem;
import kd.bos.list.*;
import kd.bos.list.column.ListOperationColumnDesc;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.MetadataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.sdk.plugin.Plugin;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.jetbrains.annotations.NotNull;
import q1k2.pxlx.train.common.Q1k2SurveyLwConstant;
import q1k2.pxlx.train.common.Q1k2SurveyUserLwConstant;
import q1k2.pxlx.train.common.Q1k2SurveyquestionLwConstant;
import q1k2.pxlx.train.dto.AnswerDto;
import q1k2.pxlx.train.dto.QuestionDto;
import q1k2.pxlx.train.dto.QuestionOptionDto;
import q1k2.pxlx.train.enums.QuestionTypeEnum;

import java.awt.*;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.List;

/**
 * @Description 调研管理列表插件
 * @Created 刘维
 */
public class SurveyListPlugin extends AbstractListPlugin implements Plugin {
    private final String PREVIEW = "preview";//预览
    private final String SURVEYREPORT = "surveyreport";//统计分析
    private final String PROCESS = "process";//过程管理
    private final String EXPORTSURVEY = "exportsurvey";//导出明细
    private final String SURVEY_PREVIEW = "q1k2_survey_preview_lw";//调研与评估预览单据标识
    private final String SURVEY_REPORT = "q1k2_survey_report_lw";//调研统计分析单据标识
    private final String SURVEY_USER = "q1k2_survey_user_lw";//调研评估用户单据标识

    /**
     * 数据包装事件,用于修改发送给前端之前的数据，用于单元格填值
     *
     * @param evt 数据包装事件参数 PackageDataEvent
     */
    @Override
    public void packageData(PackageDataEvent evt) {
        super.packageData(evt);
        this.setOperationColItemStyle(evt);
    }

    /**
     * 操作列控制(某个操作项上显示名称)
     *
     * @param evt
     */
    private void setOperationColItemStyle(PackageDataEvent evt) {
        Object obj = evt.getSource();
        // 获取当前行数据
        DynamicObject rowData = evt.getRowData();
        // 判断是否是操作列
        if (obj instanceof ListOperationColumnDesc) {
            String billstatus = rowData.getString("billstatus");// 状态
            List<OperationColItem> operationColItems = (List<OperationColItem>) evt.getFormatValue();
            for (OperationColItem item : operationColItems) {
                //操作键
                String operationKey = item.getOperationKey();
                if (("modify".equalsIgnoreCase(operationKey) || "preview".equalsIgnoreCase(operationKey)) && "A".equals(billstatus)) {
                    //暂存状态 显示 修改和预览
                    item.setVisible(true);
                    //item.setLocked(true);
                } else if (("process".equalsIgnoreCase(operationKey) || "surveyreport".equalsIgnoreCase(operationKey) || "share".equalsIgnoreCase(operationKey) || "exportsurvey".equalsIgnoreCase(operationKey)) && "B".equals(billstatus)) {
                    //已提交状态 显示 过程管理、统计分析、分享、导出明细
                    item.setVisible(true);
                }
            }
        }
    }


    @Override
    public void afterDoOperation(AfterDoOperationEventArgs args) {
        super.afterDoOperation(args);
        //预览
        if (StringUtils.equals(PREVIEW, args.getOperateKey())) {
            //构造FormShowParameter，打开子页面弹窗
            FormShowParameter showParameter = new FormShowParameter();//构造对象
            IListView listView = (IListView) this.getView();
            //获取选中行数据
            ListSelectedRow selectedRow = listView.getCurrentSelectedRowInfo();
            Object primaryKeyValue = selectedRow.getPrimaryKeyValue();
            //查询调研管理数据
            DynamicObject surveyInfo = BusinessDataServiceHelper.loadSingle(primaryKeyValue, Q1k2SurveyLwConstant.FORMBILLID);
            //自定义参数
            Map<String, Object> customParameters = new HashMap<String, Object>();
            String contentJson = surveyInfo.getString(Q1k2SurveyLwConstant.Q1K2_CONTENT_TAG);
            List<QuestionDto> questionList = JSON.parseArray(contentJson, QuestionDto.class);
            customParameters.put("name", surveyInfo.get(Q1k2SurveyLwConstant.Q1K2_NAME));
            customParameters.put("preface", surveyInfo.get(Q1k2SurveyLwConstant.Q1K2_PREFACE));
            customParameters.put("questionData", questionList);
            customParameters.put("surveyType", "SURVEY");
            showParameter.setCustomParams(customParameters);
            OpenStyle openStyle = showParameter.getOpenStyle();
            openStyle.setShowType(ShowType.Modal);//设置打开方式
            showParameter.setFormId(SURVEY_PREVIEW);//设置子页面Formld
            this.getView().showForm(showParameter);//通过showForm打开新页面
        } else if (StringUtils.equals(SURVEYREPORT, args.getOperateKey())) {//统计与分析
            //构造FormShowParameter，打开子页面弹窗
            FormShowParameter showParameter = new FormShowParameter();//构造对象
            IListView listView = (IListView) this.getView();
            //获取选中行数据
            ListSelectedRow selectedRow = listView.getCurrentSelectedRowInfo();
            showParameter.setCustomParam("surveyId", selectedRow.getPrimaryKeyValue());
            OpenStyle openStyle = showParameter.getOpenStyle();
            openStyle.setShowType(ShowType.Modal);//设置打开方式
            showParameter.setFormId(SURVEY_REPORT);//设置子页面Formld
            this.getView().showForm(showParameter);//通过showForm打开新页面
        } else if (StringUtils.equals(PROCESS, args.getOperateKey())) {//过程管理
            IListView listView = (IListView) this.getView();
            ListSelectedRow currentSelectedRowInfo = listView.getCurrentSelectedRowInfo();
            Object primaryKeyValue = currentSelectedRowInfo.getPrimaryKeyValue();
            if (currentSelectedRowInfo != null) {
                ListShowParameter listShowParameter = new ListShowParameter();
                //设置FormId，列表的FormId固定为"bos_list"
                listShowParameter.setFormId("bos_list");
                //设置BillFormId，为列表所对应单据的标识
                listShowParameter.setBillFormId(Q1k2SurveyUserLwConstant.FORMBILLID);
                //设置弹出页面标题
                //listShowParameter.setCaption("选择请假类型：");
                //设置弹出页面的打开方式
                listShowParameter.getOpenStyle().setShowType(ShowType.Modal);
                //设置弹出页面的样式
                //设置为不能多选，如果为true则表示可以多选
                //listShowParameter.setMultiSelect(false);
                //设置页面回调事件方法
                //listShowParameter.setCloseCallBack(new CloseCallBack(this, SURVEY_USER));
                ListFilterParameter listFilterParameter = new ListFilterParameter();
                listFilterParameter.setFilter(new QFilter(Q1k2SurveyUserLwConstant.Q1K2_SURVEY, QCP.equals, primaryKeyValue));
                listShowParameter.setListFilterParameter(listFilterParameter);
                listShowParameter.setCustomParam(Q1k2SurveyUserLwConstant.Q1K2_SURVEY, primaryKeyValue);
                //绑定子页面到当前页面
                this.getView().showForm(listShowParameter);
            }
        } else if (StringUtils.equals(EXPORTSURVEY, args.getOperateKey())) {//导出明细
            IListView listView = (IListView) this.getView();
            ListSelectedRow currentSelectedRowInfo = listView.getCurrentSelectedRowInfo();
            Object primaryKeyValue = currentSelectedRowInfo.getPrimaryKeyValue();
            if (currentSelectedRowInfo != null) {
                BillList billList = getControl("billlistap");
                List<IListColumn> listColumns = billList.getShowListColumns();
                ListSelectedRowCollection selectedRows = billList.getSelectedRows();
                //EntityType entityType = billList.getEntityType();
                String entityName = billList.getBillFormId();
                // 通过标识名称，获取单据的中文名
                String title = "行首标题：" + MetadataServiceHelper.getDataEntityType(entityName).getDisplayName().toString();
                // 字段标识列表
                List<String> selectFieldList = new ArrayList<>();
                // excelList用于提供数据源给 excel工作簿
                List<List<String>> excelList = new ArrayList<>();
                // 标题列表
                List<String> captionList = new ArrayList<>();
                for (IListColumn column : listColumns) {
                    if (column.getEntityName() != null && !column.getListFieldKey().equals("fseq")) {
                        captionList.add(column.getCaption().toString());
                        selectFieldList.add(getFieldKey(entityName, column));
                    }
                }
                excelList.add(captionList);
                // 查询数据
                DynamicObjectCollection dynamicObjectCollection = queryData(entityName, listColumns, selectedRows);
                // 把字段的值添加进excelList，excelList之后用来创建XSSFWorkbook
                for (DynamicObject dynamicObject : dynamicObjectCollection) {
                    List<String> list = new ArrayList<>();
                    // 用字段标识列表selectFieldList 从dynamicObject中获取数据
                    for (String filed : selectFieldList) {
                        list.add(dynamicObject.get(filed).toString());
                    }
                    excelList.add(list);
                }

                //String contentJson = survey.getString(Q1k2SurveyLwConstant.Q1K2_CONTENT_TAG);
                //List<QuestionDto> questionDtos = JSON.parseArray(contentJson, QuestionDto.class);
                // 创建excel工作簿
                //XSSFWorkbook workbook = excel(questionDtos, excelList, title);
                // 上传
                //String path = upload(entityName, workbook);
                // 弹出提示框
                //getView().showMessage("下载文件链接：" + RequestContext.get().getClientFullContextPath() + "/attachment/download.do?path=" + path);
                //getView().openUrl(RequestContext.get().getClientFullContextPath() + "/attachment/download.do?path=" + path);
            }
        }
    }


    /**
     * 试题对象转试题实体对象
     *
     * @return
     */
    @NotNull
    private List<QuestionDto> getQuestionDtos(Object primaryKeyValue, XSSFSheet sheet) {
        String selectFields = "";
        DynamicObject[] questions = BusinessDataServiceHelper.load(
                Q1k2SurveyquestionLwConstant.FORMBILLID,
                selectFields,
                new QFilter(Q1k2SurveyquestionLwConstant.Q1K2_SURVEY, QCP.equals, primaryKeyValue).toArray());
        List<QuestionDto> questionDtos = new ArrayList<>();
        //根据调研id查询调研用户
        DynamicObject[] surveyUsers = BusinessDataServiceHelper.load(
                Q1k2SurveyUserLwConstant.FORMBILLID,
                selectFields,
                new QFilter(Q1k2SurveyUserLwConstant.Q1K2_SURVEY, QCP.equals, primaryKeyValue).toArray());
        // 创建标题
        XSSFRow questionContentRow = sheet.createRow(0);
        XSSFRow questionOptionRow = sheet.createRow(1);
        XSSFCell questionContentCell1 = questionContentRow.createCell(0);
        questionContentCell1.setCellValue("题目");
        //合并
        sheet.addMergedRegion(new CellRangeAddress(0, 1, 0, 0));
        XSSFCell questionContentCell2 = questionContentRow.createCell(1);
        questionContentCell2.setCellValue("题干");
        sheet.addMergedRegion(new CellRangeAddress(0, 0, 1, 6));

        XSSFCell questionOptionCell1 = questionOptionRow.createCell(1);
        questionOptionCell1.setCellValue("选项");
        sheet.addMergedRegion(new CellRangeAddress(1, 1, 1, 6));
        //生成题目表头
        XSSFRow contentRow = sheet.createRow(2);
        XSSFCell namecell = contentRow.createCell(0);
        namecell.setCellValue("姓名/统计");
        sheet.addMergedRegion(new CellRangeAddress(2, 2, 0, 1));
        XSSFCell accountCell = contentRow.createCell(2);
        accountCell.setCellValue("账号");
        XSSFCell depNameCell = contentRow.createCell(3);
        depNameCell.setCellValue("部门全称");
        XSSFCell jobClanCell = contentRow.createCell(4);
        jobClanCell.setCellValue("职务族");
        XSSFCell jobCell = contentRow.createCell(5);
        jobCell.setCellValue("职务");
        XSSFCell positionCell = contentRow.createCell(6);
        positionCell.setCellValue("职位");
        int cellIndex = 7;
        for (int i = 0; questions != null && i < questions.length; i++) {
            DynamicObject question = questions[i];
            //题目序号
            Integer seq = question.getInt(Q1k2SurveyquestionLwConstant.Q1K2_SEQ);
            //题目
            String content = question.getString(Q1k2SurveyquestionLwConstant.Q1K2_CONTENT);
            //类型名称
            String typeName = QuestionTypeEnum.getEnum(question.getString(Q1k2SurveyquestionLwConstant.Q1K2_TYPE)).getName();
            XSSFCell questionContentCell = questionContentRow.createCell(i + cellIndex);
            questionContentCell.setCellValue(seq + "、" + content + "(" + typeName + ")");
            //选项
            DynamicObjectCollection options = question.getDynamicObjectCollection(Q1k2SurveyquestionLwConstant.ENTRYENTITY);
            for (int j = 0; options != null && j < options.size(); j++) {
                DynamicObject option = options.get(j);
                String optionDtoContent = option.getString(Q1k2SurveyquestionLwConstant.Q1K2_OPTION_CONTENT);
                //选项从A开始生成
                int optionSeq = option.getInt(Q1k2SurveyquestionLwConstant.Q1K2_OPTION_SEQ);
                char optionName = (char) (65 + optionSeq);
                //设置选项
                XSSFCell questionOptionCell = questionOptionRow.createCell(j + cellIndex);
                questionOptionCell.setCellValue(optionName + "、" + optionDtoContent);
                //设置选中该选项人数
                int selectedNum = option.getInt(Q1k2SurveyquestionLwConstant.Q1K2_SELECTED_NUM);
                XSSFCell contentCell = contentRow.createCell(j + cellIndex);
                contentCell.setCellValue(selectedNum);
                cellIndex++;
            }
            for (int j = 0; j < surveyUsers.length; j++) {
                DynamicObject surveyUser = surveyUsers[j];
                String answerJson = surveyUser.getString(Q1k2SurveyUserLwConstant.Q1K2_ANSWER_TAG);
                List<AnswerDto> answerDtos = JSON.parseArray(answerJson, AnswerDto.class);

            }

//            //id
//            String questionId = String.valueOf(question.get(Q1k2SurveyquestionLwConstant.ID));
//            questionDto.setId(questionId);
//            //题目
//            questionDto.setContent(question.getString(Q1k2SurveyquestionLwConstant.Q1K2_CONTENT));
//            //最高分
//            questionDto.setMaxScore(question.getBigDecimal(Q1k2SurveyquestionLwConstant.Q1K2_MAX_SCORE));
//            //最低分
//            questionDto.setMinScore(question.getBigDecimal(Q1k2SurveyquestionLwConstant.Q1K2_MIN_SCORE));
//            //是否必填
//            questionDto.setRequired(question.getBoolean(Q1k2SurveyquestionLwConstant.Q1K2_REQUIRED));
//            //顺序
//            questionDto.setSeq(question.getInt(Q1k2SurveyquestionLwConstant.Q1K2_SEQ));
//            //类型
//            String type = question.getString(Q1k2SurveyquestionLwConstant.Q1K2_TYPE);
//            questionDto.setType(type);
//            //类型名称
//            String typeName = QuestionTypeEnum.getEnum(type).getName();
//            questionDto.setTypeName(typeName);
//            //答案
//            if (answerDtos != null) {
//                for (AnswerDto answer : answerDtos) {
//                    if (answer.getQuestionId().equals(questionId)) {
//                        questionDto.setAnswer(answer.getAnswer());
//                        break;
//                    }
//                }
//            }
//            //选项
//            DynamicObjectCollection questionOptions = question.getDynamicObjectCollection(Q1k2SurveyquestionLwConstant.ENTRYENTITY);
//            List options = new ArrayList();
//            for (int j = 0; questionOptions != null && j < questionOptions.size(); j++) {
//                DynamicObject questionOption = questionOptions.get(j);
//                QuestionOptionDto questionOptionDto = new QuestionOptionDto();
//                questionOptionDto.setSeq(questionOption.getInt(Q1k2SurveyquestionLwConstant.Q1K2_OPTION_SEQ));
//                questionOptionDto.setId(String.valueOf(questionOption.get("id")));
//                questionOptionDto.setScore(questionOption.getBigDecimal(Q1k2SurveyquestionLwConstant.Q1K2_OPTION_SCORE));
//                questionOptionDto.setContent(questionOption.getString(Q1k2SurveyquestionLwConstant.Q1K2_OPTION_CONTENT));
//                options.add(questionOptionDto);
//            }
//            questionDto.setOptions(options);
//            questionDtos.add(questionDto);
        }
        return questionDtos;
    }

    /**
     * @param entityName   单据标识
     * @param listColumns  列
     * @param selectedRows 已选行
     * @return
     */
    private DynamicObjectCollection queryData(String entityName, List<IListColumn> listColumns, ListSelectedRowCollection selectedRows) {
        StringBuilder stringBuilder = new StringBuilder();
        // 构造查询字段
        for (int i = 0; i < listColumns.size(); i++) {
            IListColumn listColumn = listColumns.get(i);
            if (listColumn.getEntityName() != null && !listColumn.getListFieldKey().equals("fseq")) {
                if (i == listColumns.size() - 1) {
                    stringBuilder.append(getFieldKey(entityName, listColumn));
                } else {
                    stringBuilder.append(getFieldKey(entityName, listColumn) + ", ");
                }
            }
        }
        String selectFields = stringBuilder.toString();
        // 构造idList，获取已选行的id，之后作为过滤条件去查询数据
        List<Long> idList = new ArrayList<>();
        for (ListSelectedRow row : selectedRows) {
            idList.add((Long) row.getPrimaryKeyValue());
        }
        // QFilter的过滤条件为：单据id等于idList里面的这些id才符合获取条件，其他的数据不获取
        QFilter qFilter = new QFilter("id", QFilter.in, idList);
        // 查询数据
        DynamicObjectCollection dynamicObjectCollection = QueryServiceHelper.query(entityName, selectFields, qFilter.toArray());
            /*DynamicObject[] dynamicObjectArray =
                    BusinessDataServiceHelper.load(entityName, selectFields, qFilter.toArray());*/
        return dynamicObjectCollection;
    }

    private String getFieldKey(String mainEntityName, IListColumn listColumns) {
        if (listColumns.getEntityName().equals(mainEntityName)) {
            return listColumns.getListFieldKey();
        } else {
            return listColumns.getEntityName() + "." + listColumns.getListFieldKey();
        }
    }

    /**
     * excel生成
     *
     * @param excel 数据源，List<List<String>>，外层的list是行，里面的List<String>是列，存储value
     *              excel.size()是行数，excel.get(n).size()是列数
     * @return
     */
    private XSSFWorkbook excel(List<QuestionDto> questionDtos, List<List<String>> excel, String title) {
        //创建excel工作簿
        XSSFWorkbook workbook = new XSSFWorkbook();
        //创建工作表sheet
        XSSFSheet sheet = workbook.createSheet();
        //设置默认列宽
        sheet.setDefaultColumnWidth(20);


        // 设置首行标题的一些风格样式
        XSSFCellStyle titleStyle = workbook.createCellStyle();
        // 设置背景色
        //设置填充方案
        titleStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        // 正红色
        //setBorder.setFillForegroundColor(new XSSFColor(Color.RED));
        // 设置自定义填充颜色，天蓝色
        titleStyle.setFillForegroundColor(new XSSFColor(new Color(135, 206, 250), new DefaultIndexedColorMap()));
        // 设置水平居中
        titleStyle.setAlignment(HorizontalAlignment.CENTER);
        // 设置垂直居中
        titleStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        // 设置字
        XSSFFont font = workbook.createFont();
        // 字体颜色橘红色
        font.setColor(new XSSFColor(new Color(255, 69, 0), new DefaultIndexedColorMap()));
        // 设置字号
        font.setFontHeight(20);
        // 设置字体
        font.setFontName("微软雅黑");
        titleStyle.setFont(font);
        //headCell.setCellStyle(titleStyle);
        // 合并第1行的前几列，合并列数 = excel的列数
        //CellRangeAddress titleCellAddresses = new CellRangeAddress(0, 0, 0, excel.get(0).size() - 1);
        //sheet.addMergedRegion(titleCellAddresses);
        // 单据列表数据风格样式，设置字体为黑体，字号15
        XSSFCellStyle billStyle = workbook.createCellStyle();
        XSSFFont billFont = workbook.createFont();
        billFont.setFontName("黑体");
        billFont.setFontHeight(15);
        billStyle.setFont(billFont);
        //写入单据列表数据
//        for (int i = 0; i < excel.size(); i++) {
//            // i+1是因为前面第1行加了一个标题，单据列表数据是从Excel的第2行开始的，所以要+1
//            XSSFRow nrow = sheet.createRow(i + 1);
//            for (int u = 0; u < excel.get(i).size(); u++) {
//                XSSFCell ncell = nrow.createCell(u);
//                ncell.setCellStyle(billStyle);
//                ncell.setCellValue(excel.get(i).get(u));
//            }
//        }
        return workbook;
    }

    /**
     * @param entityName
     * @param workbook
     * @return path 路径
     */
    private String upload(String entityName, XSSFWorkbook workbook) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
        String fileName = entityName + sdf.format(new Date()) + ".xlsx";
        String pathName = "/offices/" + fileName;
        try {
            OutputStream outputStream = new ByteArrayOutputStream();
            workbook.write(outputStream);
            InputStream inputStream = parse(outputStream);
            FileService fs = FileServiceFactory.getAttachmentFileService();
            String path = fs.upload(new FileItem(fileName, pathName, inputStream));
            return path;
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        return "";
    }

    /**
     * outputStream转inputStream
     *
     * @param out outputStream
     * @return inputStream
     * @throws Exception
     */
    public ByteArrayInputStream parse(final OutputStream out) throws Exception {
        ByteArrayOutputStream baos = (ByteArrayOutputStream) out;
        final ByteArrayInputStream swapStream = new ByteArrayInputStream(baos.toByteArray());
        return swapStream;
    }
}