package q1k2.pxlx.train.plugin.form;

import com.alibaba.fastjson.JSON;
import com.kingdee.util.StringUtils;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.OrmLocaleValue;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.events.PackageDataEvent;
import kd.bos.form.FormShowParameter;
import kd.bos.form.OpenStyle;
import kd.bos.form.ShowType;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.operatecol.OperationColItem;
import kd.bos.list.IListView;
import kd.bos.list.column.ListOperationColumnDesc;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.sdk.plugin.Plugin;
import q1k2.pxlx.train.common.Q1k2SurveytmplLwConstant;
import q1k2.pxlx.train.dto.QuestionDto;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description 调研与评估模板列表插件
 * @Created 刘维
 */
public class SurveyTempleListPlugin extends AbstractListPlugin implements Plugin {
    private final String PREVIEW = "preview";//预览
    private final String SURVEY_PREVIEW = "q1k2_survey_preview_lw";//调研与评估预览单据标识


    /**
     * 数据包装事件,用于修改发送给前端之前的数据，用于单元格填值
     *
     * @param evt 数据包装事件参数 PackageDataEvent
     */
    @Override
    public void packageData(PackageDataEvent evt) {
        super.packageData(evt);
        this.setOperationColItemStyle(evt);
    }

    /**
     * 操作列控制(某个操作项上显示名称)
     *
     * @param evt
     */
    private void setOperationColItemStyle(PackageDataEvent evt) {
        Object obj = evt.getSource();
        // 获取当前行数据
        DynamicObject rowData = evt.getRowData();
        // 判断是否是操作列
        if (obj instanceof ListOperationColumnDesc) {
            String billstatus = rowData.getString("status");// 状态
            List<OperationColItem> operationColItems = (List<OperationColItem>) evt.getFormatValue();
            for (OperationColItem item : operationColItems) {
                //操作键
                String operationKey = item.getOperationKey();
                if (("modify".equalsIgnoreCase(operationKey)
                        || "submit".equalsIgnoreCase(operationKey))
                        && "A".equals(billstatus)) {
                    //暂存状态 显示 修改和预览
                    item.setVisible(true);
                    //item.setLocked(true);
                } else if (("unsubmit".equalsIgnoreCase(operationKey)
                        || "preview".equalsIgnoreCase(operationKey))
                        && "B".equals(billstatus)) {
                    //已提交状态 显示 过程管理、统计分析、分享、导出明细
                    item.setVisible(true);
                }
            }
        }
    }

    @Override
    public void afterDoOperation(AfterDoOperationEventArgs event) {
        super.afterDoOperation(event);
        if (StringUtils.equals(PREVIEW, event.getOperateKey())) {//预览
            //构造FormShowParameter，打开子页面弹窗
            FormShowParameter showParameter = new FormShowParameter();//构造对象
            showParameter.setCaption("试题模板预览");
            IListView listView = (IListView) this.getView();
            //获取选中行数据
            ListSelectedRow selectedRow = listView.getCurrentSelectedRowInfo();
            Object primaryKeyValue = selectedRow.getPrimaryKeyValue();
            //查询模板数据
            DynamicObject surveyTemplateInfo = BusinessDataServiceHelper.loadSingle(primaryKeyValue, Q1k2SurveytmplLwConstant.FORMBILLID);
            //自定义参数
            Map<String, Object> customParameters = new HashMap<String, Object>();
            String contentJson = surveyTemplateInfo.getString(Q1k2SurveytmplLwConstant.Q1K2_CONTENT_TAG);
            List<QuestionDto> questionList = JSON.parseArray(contentJson, QuestionDto.class);
            OrmLocaleValue name = (OrmLocaleValue) surveyTemplateInfo.getLocaleString(Q1k2SurveytmplLwConstant.NAME);
            customParameters.put("name", name.getLocaleValue());
            customParameters.put("preface", surveyTemplateInfo.get(Q1k2SurveytmplLwConstant.Q1K2_PREFACE));
            customParameters.put("questionData", questionList);
            customParameters.put("surveyType", "SURVEY");
            showParameter.setCustomParams(customParameters);
            OpenStyle openStyle = showParameter.getOpenStyle();
            openStyle.setShowType(ShowType.Modal);//设置打开方式
            showParameter.setFormId(SURVEY_PREVIEW);//设置子页面Formld
            this.getView().showForm(showParameter);//通过showForm打开新页面
        }
    }
}