package q1k2.pxlx.train.plugin.form;

import com.kingdee.util.StringUtils;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.form.CloseCallBack;
import kd.bos.form.ShowFormHelper;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.list.ListFilterParameter;
import kd.bos.list.ListShowParameter;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;
import q1k2.pxlx.train.common.Q1k2SurveyLwConstant;
import q1k2.pxlx.train.common.Q1k2SurveyUserLwConstant;

import java.math.BigDecimal;
import java.util.*;

/**
 * @Description 调研评估用户列表插件
 * @Created 刘维
 */
public class SurveyUserListPlugin extends AbstractListPlugin implements Plugin {
    private final String BTN_ADDUSER = "q1k2_btn_adduser";//增加用户
    private final String DELETE = "delete";//删除用户
    //人员单据标识
    private final String BOS_USER = "bos_user";



    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        if (StringUtils.equals(BTN_ADDUSER, evt.getItemKey())) {//新增用户
            //第一个参数为列表的单据标识，第二个参数为是否支持多选
            ListShowParameter showParameter = ShowFormHelper.createShowListForm(BOS_USER, true);
            //获取页面跳转参数
            ListShowParameter formShowParameter = (ListShowParameter) this.getView().getFormShowParameter();
            ListFilterParameter filterParameter = formShowParameter.getListFilterParameter();
            List<QFilter> listQFilters = filterParameter.getQFilters();
            //查询调研评估用户数据
            DynamicObject[] users = BusinessDataServiceHelper.load(Q1k2SurveyUserLwConstant.FORMBILLID, "q1k2_user", listQFilters.toArray(new QFilter[]{}));
            Set set = new HashSet();
            for (int i = 0; i < users.length; i++) {
                DynamicObject user = users[i].getDynamicObject(Q1k2SurveyUserLwConstant.Q1K2_USER);
                set.add(user.getPkValue());
            }
            List<QFilter> qFilters = new ArrayList<QFilter>();
            //移除掉已经选中的人员
            qFilters.add(new QFilter("id", QCP.not_in, set));
            ListFilterParameter listFilterParameter = new ListFilterParameter(qFilters, null);
            showParameter.setListFilterParameter(listFilterParameter);
            showParameter.setCloseCallBack(new CloseCallBack(this, BTN_ADDUSER));
            //打开F7窗口
            this.getView().showForm(showParameter);
        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent event) {
        super.closedCallBack(event);
        if (StringUtils.equals(BTN_ADDUSER, event.getActionId())) {
            //新增用户回调
            ListSelectedRowCollection returnData = (ListSelectedRowCollection) event.getReturnData();
            if (returnData != null && returnData.size() > 0) {
                //查询根据id人员数据
                QFilter qFilter = new QFilter("id", QCP.in, returnData.getPrimaryKeyValues());
                DynamicObject[] users = BusinessDataServiceHelper.load(BOS_USER, "id,name,billno", qFilter.toArray());
                DynamicObject[] surveyUsers = new DynamicObject[users.length];
                //调研管理主键
                Object surveyPk = this.getView().getFormShowParameter().getCustomParam(Q1k2SurveyUserLwConstant.Q1K2_SURVEY);
                for (int j = 0; j < users.length; j++) {
                    //创建调研评估用户对象
                    DynamicObject surveyUserInfo = BusinessDataServiceHelper.newDynamicObject(Q1k2SurveyUserLwConstant.FORMBILLID);
                    //调研ID
                    surveyUserInfo.set(Q1k2SurveyUserLwConstant.Q1K2_SURVEY, surveyPk);
                    //用户
                    surveyUserInfo.set(Q1k2SurveyUserLwConstant.Q1K2_USER, users[j]);
                    //单据状态
                    surveyUserInfo.set(Q1k2SurveyUserLwConstant.Q1K2_STATUS, "UNFINISH");
                    surveyUsers[j] = surveyUserInfo;
                }
                //保存
                SaveServiceHelper.saveOperate(Q1k2SurveyUserLwConstant.FORMBILLID, surveyUsers, OperateOption.create());
                //更新调研管理用户列表
                DynamicObject survey = BusinessDataServiceHelper.loadSingle(surveyPk, Q1k2SurveyLwConstant.FORMBILLID);
                DynamicObjectCollection surveyRangeUsers = survey.getDynamicObjectCollection(Q1k2SurveyLwConstant.Q1K2_SURVEY_RANGE_USER);
                for (int j = 0; j < users.length; j++) {
                    DynamicObject surveyUser = surveyRangeUsers.addNew();
                    //用户
                    surveyUser.set(Q1k2SurveyLwConstant.Q1K2_RANGE_USER, users[j]);
                }
                //参与人数
                int joinNum = surveyRangeUsers.size();
                survey.set(Q1k2SurveyLwConstant.Q1K2_JOIN_NUM, joinNum);
                //完成人数
                int finishNum = survey.getInt(Q1k2SurveyLwConstant.Q1K2_FINISH_NUM);
                //完成情况
                BigDecimal finishNumDecimal = new BigDecimal(finishNum);
                BigDecimal joinNumDecimal = new BigDecimal(joinNum);
                BigDecimal finishRate = finishNumDecimal.divide(joinNumDecimal, 2, BigDecimal.ROUND_HALF_UP);
                survey.set(Q1k2SurveyLwConstant.Q1K2_FINISH_RATE, finishRate);
                SaveServiceHelper.save(new DynamicObject[]{survey});
                //刷新
                this.getView().updateView();
            }
        }
    }
}