package q1k2.pxlx.train.plugin.operate;

import com.alibaba.fastjson.JSON;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.DeleteServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;
import q1k2.pxlx.train.common.Q1k2SurveyLwConstant;
import q1k2.pxlx.train.common.Q1k2SurveyUserLwConstant;
import q1k2.pxlx.train.common.Q1k2SurveyquestionLwConstant;
import q1k2.pxlx.train.dto.QuestionDto;
import q1k2.pxlx.train.dto.QuestionOptionDto;

import java.util.List;

/**
 * 单据操作插件
 */
public class SurveyOperationServicePlugIn extends AbstractOperationServicePlugIn implements Plugin {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add(Q1k2SurveyLwConstant.Q1K2_CONTENT_TAG);
        fieldKeys.add(Q1k2SurveyLwConstant.Q1K2_SURVEY_RANGE_USER);
        fieldKeys.add(Q1k2SurveyLwConstant.Q1K2_RANGE_USER);
    }

    /**
     * 操作校验通过之后，开启事务之前，触发此事件；
     *
     * @remark 插件可以在此事件，对通过校验的数据，进行整理
     */
    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        String operationKey = e.getOperationKey();
        DynamicObject[] dataEntities = e.getDataEntities();
        if ("save".equalsIgnoreCase(operationKey)) {//保存
            for (int i = 0; i < dataEntities.length; i++) {
                DynamicObject surveyInfo = dataEntities[i];
                //人员单据体
                DynamicObjectCollection userEntitys = surveyInfo.getDynamicObjectCollection(Q1k2SurveyLwConstant.Q1K2_SURVEY_RANGE_USER);
                //更新人员
                surveyInfo.set(Q1k2SurveyLwConstant.Q1K2_JOIN_NUM, userEntitys.size());
            }
        }
    }

    @Override
    public void endOperationTransaction(EndOperationTransactionArgs e) {
        super.endOperationTransaction(e);
        String operationKey = e.getOperationKey();
        DynamicObject[] dataEntities = e.getDataEntities();
        if ("save".equals(operationKey)) {//保存
            //判断
            for (int i = 0; i < dataEntities.length; i++) {
                DynamicObject surveyInfo = dataEntities[i];
                Object pkValue = surveyInfo.getPkValue();
                //根据主键查询调研管理数据
                //surveyInfo = BusinessDataServiceHelper.loadSingle(pkValue, Q1k2SurveyLwConstant.FORMBILLID);
                //试题列表
                String context = surveyInfo.getString(Q1k2SurveyLwConstant.Q1K2_CONTENT_TAG);
                List<QuestionDto> questionDtos = JSON.parseArray(context, QuestionDto.class);
                //删除调研试题数据
                QFilter qFilter = new QFilter(Q1k2SurveyquestionLwConstant.Q1K2_SURVEY, QCP.equals, pkValue);
                int delete = DeleteServiceHelper.delete(Q1k2SurveyquestionLwConstant.FORMBILLID, qFilter.toArray());
                //新增调研试题
                DynamicObject[] questions = new DynamicObject[questionDtos.size()];
                for (int j = 0; j < questionDtos.size(); j++) {
                    QuestionDto questionDto = questionDtos.get(j);
                    DynamicObject question = BusinessDataServiceHelper.newDynamicObject(Q1k2SurveyquestionLwConstant.FORMBILLID);
                    //题目
                    question.set(Q1k2SurveyquestionLwConstant.Q1K2_CONTENT, questionDto.getContent());
                    //调研ID
                    question.set(Q1k2SurveyquestionLwConstant.Q1K2_SURVEY, pkValue);
                    //类型
                    question.set(Q1k2SurveyquestionLwConstant.Q1K2_TYPE, questionDto.getType());
                    //必填
                    question.set(Q1k2SurveyquestionLwConstant.Q1K2_REQUIRED, questionDto.getRequired());
                    //最大分数
                    question.set(Q1k2SurveyquestionLwConstant.Q1K2_MAX_SCORE, questionDto.getMaxScore());
                    //最小分数
                    question.set(Q1k2SurveyquestionLwConstant.Q1K2_MIN_SCORE, questionDto.getMinScore());
                    //排序
                    question.set(Q1k2SurveyquestionLwConstant.Q1K2_SEQ, questionDto.getSeq());
                    //单据体
                    DynamicObjectCollection questionOptions = question.getDynamicObjectCollection(Q1k2SurveyquestionLwConstant.ENTRYENTITY);
                    List<QuestionOptionDto> options = questionDto.getOptions();
                    for (int k = 0; options != null && k < options.size(); k++) {
                        QuestionOptionDto questionOptionDto = options.get(k);
                        DynamicObject option = questionOptions.addNew();
                        //内容
                        option.set(Q1k2SurveyquestionLwConstant.Q1K2_OPTION_CONTENT, questionOptionDto.getContent());
                        //分数
                        option.set(Q1k2SurveyquestionLwConstant.Q1K2_OPTION_SCORE, questionOptionDto.getScore());
                        //排序
                        option.set(Q1k2SurveyquestionLwConstant.Q1K2_OPTION_SEQ, questionOptionDto.getSeq());
                    }
                    questions[j] = question;
                }
                SaveServiceHelper.saveOperate(Q1k2SurveyquestionLwConstant.FORMBILLID, questions, OperateOption.create());
            }
        } else if ("submit".equals(operationKey)) {//提交
            //判断
            for (int i = 0; i < dataEntities.length; i++) {
                DynamicObject surveyInfo = dataEntities[i];
                Object pkValue = surveyInfo.getPkValue();
                //根据主键查询调研管理数据
                //surveyInfo = BusinessDataServiceHelper.loadSingle(pkValue, Q1k2SurveyLwConstant.FORMBILLID);
                //人员单据体
                DynamicObjectCollection userEntitys = surveyInfo.getDynamicObjectCollection(Q1k2SurveyLwConstant.Q1K2_SURVEY_RANGE_USER);
                //存放调研评估用户对象
                DynamicObject[] surveyUsers = new DynamicObject[userEntitys.size()];
                for (int j = 0; j < userEntitys.size(); j++) {
                    DynamicObject userInfo = userEntitys.get(j).getDynamicObject(Q1k2SurveyLwConstant.Q1K2_RANGE_USER);
                    //创建调研评估用户对象
                    DynamicObject surveyUserInfo = BusinessDataServiceHelper.newDynamicObject(Q1k2SurveyUserLwConstant.FORMBILLID);
                    //调研ID
                    surveyUserInfo.set(Q1k2SurveyUserLwConstant.Q1K2_SURVEY, pkValue);
                    //用户
                    surveyUserInfo.set(Q1k2SurveyUserLwConstant.Q1K2_USER, userInfo);
                    //单据状态
                    surveyUserInfo.set(Q1k2SurveyUserLwConstant.Q1K2_STATUS, "UNFINISH");
                    surveyUsers[j] = surveyUserInfo;
                }
                //保存
                SaveServiceHelper.saveOperate(Q1k2SurveyUserLwConstant.FORMBILLID, surveyUsers, OperateOption.create());
            }
        } else if ("unsubmit".equals(operationKey)) {
            //撤销
            for (int i = 0; i < dataEntities.length; i++) {
                DynamicObject surveyInfo = dataEntities[i];
                QFilter qFilter = new QFilter(Q1k2SurveyUserLwConstant.Q1K2_SURVEY, QCP.equals, surveyInfo.getPkValue());
                //删除调研评估对象数据
                DeleteServiceHelper.delete(Q1k2SurveyUserLwConstant.FORMBILLID, qFilter.toArray());
                //清空完成情况
                DynamicObject survey = BusinessDataServiceHelper.loadSingle(surveyInfo.getPkValue(), Q1k2SurveyLwConstant.FORMBILLID);
                //重置完成数量
                survey.set(Q1k2SurveyLwConstant.Q1K2_FINISH_NUM, 0);
                //重置完成率
                survey.set(Q1k2SurveyLwConstant.Q1K2_FINISH_RATE, 0);
                SaveServiceHelper.saveOperate(Q1k2SurveyLwConstant.FORMBILLID, new DynamicObject[]{survey}, OperateOption.create());
            }
        }
    }
}