package q1k2.pxlx.user.plugin.form;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.form.FormShowParameter;
import kd.bos.form.OpenStyle;
import kd.bos.form.ShowType;
import kd.bos.form.cardentry.CardEntry;
import kd.bos.form.control.Button;
import kd.bos.form.control.events.RowClickEvent;
import kd.bos.form.control.events.RowClickEventListener;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;
import kd.bos.util.StringUtils;
import kd.sdk.plugin.Plugin;
import org.jetbrains.annotations.NotNull;
import q1k2.pxlx.train.common.Q1k2SurveyLwConstant;
import q1k2.pxlx.train.common.Q1k2SurveyUserLwConstant;

import java.util.*;

/**
 * @Description 调研评估个人中心插件
 * @Created 刘维
 */
public class SurveyListPlugin extends AbstractFormPlugin implements Plugin, RowClickEventListener {
    private final String BTN_FINISH = "q1k2_btn_finish";//已完成
    private final String BTN_NOT_START = "q1k2_btn_not_start";//未开始

    private final String BTN_SEARCH = "q1k2_btn_search";//搜索
    private final String Q1K2_LIST = "q1k2_list";//卡片单据标识
    private final String USER_SURVEY = "q1k2_user_survey_lw";//调研评估详情标识

    private boolean isFinish = false;

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        //更新单据体数据
        updateEntityData("UNFINISH", null);//状态等于未完成
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        //给按钮增加监听
        Button notStartBtn = this.getControl(BTN_NOT_START);
        notStartBtn.addClickListener(this);
        Button finishBtn = this.getControl(BTN_FINISH);
        finishBtn.addClickListener(this);
        Button searchBtn = this.getControl(BTN_SEARCH);
        searchBtn.addClickListener(this);
        //this.addClickListeners(BTN_NOT_START, BTN_FINISH, BTN_SEARCH);
        //给单据体增加点击事件
        CardEntry entryGrid = this.getControl(Q1K2_LIST);
        entryGrid.addRowClickListener(this);
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        String key = null;
        if (evt.getSource() instanceof Button) {
            key = ((Button) evt.getSource()).getKey();
        }
        String searchValue = (String) this.getModel().getValue("q1k2j_search_name");
        if (BTN_NOT_START.equals(key)) {//未开始
            isFinish = false;
            //设置未开始页签样式
            this.getView().updateControlMetadata(BTN_NOT_START, getSelectedStyle());
            //设置已完成页签样式
            this.getView().updateControlMetadata(BTN_FINISH, getUnSelectedStyle());
            //更新单据体数据
            updateEntityData("UNFINISH", searchValue);//状态等于未完成
        } else if (BTN_FINISH.equals(key)) {//已完成
            isFinish = true;
            //设置未开始页签样式
            this.getView().updateControlMetadata(BTN_NOT_START, getUnSelectedStyle());
            //设置已完成页签样式
            this.getView().updateControlMetadata(BTN_FINISH, getSelectedStyle());
            //更新单据体数据
            updateEntityData("FINISH", searchValue);//状态等于已完成
        } else if ("q1k2_btn_search".equals(key)) {
            //搜索
            //更新单据体数据
            updateEntityData(isFinish ? "UNFINISH" : "FINISH", searchValue);
        }
    }

    /**
     * 单据体焦点行、焦点单元格切换时触发
     *
     * @param evt
     */
    @Override
    public void entryRowClick(RowClickEvent evt) {
        RowClickEventListener.super.entryRowClick(evt);
        if (evt.getRow() != -1) {
            DynamicObject entryRowEntity = this.getModel().getEntryRowEntity(Q1K2_LIST, evt.getRow());
            //调研管理id
            long surveyid = entryRowEntity.getLong("q1k2_number");
            //当前用户id
            long currentUserId = UserServiceHelper.getCurrentUserId();
            //查询调研用户数据
            List<QFilter> userFilters = new ArrayList<>();
            userFilters.add(new QFilter(Q1k2SurveyUserLwConstant.Q1K2_USER, QCP.equals, currentUserId));//用户等于当前用户
            userFilters.add(new QFilter(Q1k2SurveyUserLwConstant.Q1K2_SURVEY, QCP.equals, surveyid));//调研id
            DynamicObject surveyUserData = BusinessDataServiceHelper.loadSingle(Q1k2SurveyUserLwConstant.FORMBILLID, userFilters.toArray(new QFilter[]{}));
            //根据id查询调研管理数据
            DynamicObject survey = BusinessDataServiceHelper.loadSingle(surveyid, Q1k2SurveyLwConstant.FORMBILLID);
            FormShowParameter showParameter = new FormShowParameter();
            showParameter.setFormId(USER_SURVEY);
            OpenStyle openStyle = showParameter.getOpenStyle();
            openStyle.setShowType(ShowType.Modal);
            showParameter.setCustomParam("picture", survey.get(Q1k2SurveyLwConstant.Q1K2_COVER));
            showParameter.setCustomParam("name", survey.get(Q1k2SurveyLwConstant.Q1K2_NAME));
            showParameter.setCustomParam("beginTime", survey.get(Q1k2SurveyLwConstant.Q1K2_BEGIN_TIME));
            showParameter.setCustomParam("user", survey.getDynamicObject(Q1k2SurveyLwConstant.CREATOR).getString("name"));
            showParameter.setCustomParam("endTime", survey.get(Q1k2SurveyLwConstant.Q1K2_END_TIME));
            showParameter.setCustomParam("status", surveyUserData.get(Q1k2SurveyUserLwConstant.Q1K2_STATUS));
            showParameter.setCustomParam("surveyId", survey.get(Q1k2SurveyLwConstant.ID));
            this.getView().showForm(showParameter);
        }
    }

    /**
     * 更新单据体数据
     *
     * @param status 状态
     */
    private void updateEntityData(String status, String searchValue) {
        //当前用户id
        long currentUserId = UserServiceHelper.getCurrentUserId();
        //查询数据
        List<QFilter> userFilters = new ArrayList<>();
        userFilters.add(new QFilter(Q1k2SurveyUserLwConstant.Q1K2_USER, QCP.equals, currentUserId));//用户等于当前用户
        userFilters.add(new QFilter(Q1k2SurveyUserLwConstant.Q1K2_STATUS, QCP.equals, status));//状态
        userFilters.add(new QFilter(Q1k2SurveyUserLwConstant.Q1K2_SURVEY, QCP.is_notnull, ""));//调研id不等于空
        DynamicObject[] surveyUserDatas = BusinessDataServiceHelper.load(Q1k2SurveyUserLwConstant.FORMBILLID, "q1k2_survey", userFilters.toArray(new QFilter[]{}));
        Object[] surveyIds = new Object[surveyUserDatas.length];
        for (int i = 0; i < surveyUserDatas.length; i++) {
            surveyIds[i] = surveyUserDatas[i].get("q1k2_survey");//调研管理id
        }
        List<QFilter> qFilters = new ArrayList<>();
        qFilters.add(new QFilter("id", QCP.in, surveyIds));
        if (!StringUtils.isEmpty(searchValue)) {
            qFilters.add(QFilter.like(Q1k2SurveyLwConstant.Q1K2_NAME, "%" + searchValue + "%"));
        }
        //根据id查询调研数据(名称、图片、开始时间、结束时间)
        DynamicObject[] surveys = BusinessDataServiceHelper.load(Q1k2SurveyLwConstant.FORMBILLID,
                "id,q1k2_name,q1k2_cover,q1k2_end_time,q1k2_begin_time",
                qFilters.toArray(new QFilter[1]));
        //获取卡片单据体
        DynamicObject billObj = this.getModel().getDataEntity(true);
        DynamicObjectCollection rows = billObj.getDynamicObjectCollection(Q1K2_LIST);
        // 方法三：利用单据体属性对象EntryProp，从单据数据包中获取
        //EntryProp entryProp = (EntryProp) this.getModel().getDataEntityType().getProperties().get(Q1K2_LIST);
        //DynamicObjectCollection rows = (DynamicObjectCollection) entryProp.getValue(billObj);
        // 如下代码演示利用单据体属性对象，为单据体行增加新行
        //EntryType entryType = (EntryType) entryProp.getItemType();
        //DynamicObjectCollection rows = new DynamicObjectCollection();
        //billObj.set(Q1K2_LIST, rows);
        //删除单据体所有行
        //this.getModel().deleteEntryData(Q1K2_LIST);
        rows.clear();
        for (int i = 0; i < surveys.length; i++) {
            DynamicObject survey = surveys[i];
            //DynamicObject row = new DynamicObject(entryType);
            DynamicObject row = rows.addNew();
            row.set("id", null);
            row.set("seq", 1);
            row.set("q1k2_picturefield", survey.get(Q1k2SurveyLwConstant.Q1K2_COVER));
            row.set("q1k2_name", survey.get(Q1k2SurveyLwConstant.Q1K2_NAME));
            row.set("q1k2_begin_time", survey.get(Q1k2SurveyLwConstant.Q1K2_BEGIN_TIME));
            row.set("q1k2_end_time", survey.get(Q1k2SurveyLwConstant.Q1K2_END_TIME));
            row.set("q1k2_number", survey.get(Q1k2SurveyLwConstant.ID));
            //rows.add(row);
        }
        //刷新单据体
        this.getView().updateView(Q1K2_LIST);
    }

    /**
     * 获取未选中样式
     *
     * @return
     */
    @NotNull
    private static Map<String, Object> getUnSelectedStyle() {
        Map<String, Object> map = new HashMap<>();
        map.put("bc", "#FFFFFF");
        map.put("fc", "themeColor");
        return map;
    }

    /**
     * 获取选中样式
     *
     * @return
     */
    @NotNull
    private static Map<String, Object> getSelectedStyle() {
        Map<String, Object> map = new HashMap<>();
        map.put("bc", "themeColor");
        map.put("fc", "#FFFFFF");
        return map;
    }
}