package q1k2.pxlx.user.plugin.form;

import com.huawei.shade.com.alibaba.fastjson.JSON;
import com.huawei.shade.com.alibaba.fastjson.JSONArray;
import com.huawei.shade.com.alibaba.fastjson.JSONObject;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.ext.form.control.CustomControl;
import kd.bos.form.events.CustomEventArgs;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;
import kd.sdk.plugin.Plugin;
import q1k2.pxlx.train.common.Q1k2SurveyLwConstant;
import q1k2.pxlx.train.common.Q1k2SurveyUserLwConstant;
import q1k2.pxlx.train.common.Q1k2SurveyquestionLwConstant;

import java.math.BigDecimal;
import java.util.Date;
import java.util.EventObject;
import java.util.Map;

/**
 * 评估问卷插件动态表单插件
 *
 * @Created 刘维
 */
public class SurveyPaperPlugin extends AbstractFormPlugin implements Plugin {
    private final String Q1K2_QUESTION = "q1k2_question";//自定义控件

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        Map<String, Object> customParams = this.getView().getFormShowParameter().getCustomParams();
        CustomControl control = this.getControl(Q1K2_QUESTION);
        control.setData(customParams);
    }

    /**
     * 自定义事件
     *
     * @param e
     */
    @Override
    public void customEvent(CustomEventArgs e) {
        super.customEvent(e);
        if ("submitQuestion".equals(e.getEventName())) {
            //提交
            //答案
            String answerJson = e.getEventArgs();
            //调研管理id
            Object surveyId = this.getView().getFormShowParameter().getCustomParam("surveyId");
            //1.更新调研评估用户数据
            //当前用户
            long currentUserId = UserServiceHelper.getCurrentUserId();
            QFilter[] qFilters = new QFilter[2];
            qFilters[0] = new QFilter(Q1k2SurveyUserLwConstant.Q1K2_SURVEY, QCP.equals, surveyId);
            qFilters[1] = new QFilter(Q1k2SurveyUserLwConstant.Q1K2_USER, QCP.equals, currentUserId);
            //查询调研用户
            DynamicObject surveyUser = BusinessDataServiceHelper.loadSingle(Q1k2SurveyUserLwConstant.FORMBILLID, qFilters);
            //状态设置为已完成
            surveyUser.set(Q1k2SurveyUserLwConstant.Q1K2_STATUS, "FINISH");
            surveyUser.set(Q1k2SurveyUserLwConstant.Q1K2_ANSWER_TAG, answerJson);
            surveyUser.set(Q1k2SurveyUserLwConstant.Q1K2_ANSWER_TIME, new Date());
            //更新调研用户数据
            Object[] save = SaveServiceHelper.save(new DynamicObject[]{surveyUser});
            //2.更新试题选中情况
            JSONArray answers = JSON.parseArray(answerJson);
            for (int i = 0; i < answers.size(); i++) {
                JSONObject jsonObject = answers.getJSONObject(i);
                String questionId = jsonObject.getString("questionId");
                DynamicObject question = BusinessDataServiceHelper.loadSingle(questionId, Q1k2SurveyquestionLwConstant.FORMBILLID);
                DynamicObjectCollection options = question.getDynamicObjectCollection(Q1k2SurveyquestionLwConstant.ENTRYENTITY);
                String answerIdStr = jsonObject.getString("answer");
                BigDecimal scoreSubmit = jsonObject.getBigDecimal("score");
                String[] answerIds = answerIdStr.split(",");
                for (int j = 0; j < answerIds.length; j++) {
                    String answerId = answerIds[j];
                    for (int k = 0; k < options.size(); k++) {
                        DynamicObject option = options.get(k);
                        Long pkValue = (Long) option.getPkValue();
                        if (pkValue.toString().equals(answerId)) {
                            //被选中数
                            int selectedNum = option.getInt(Q1k2SurveyquestionLwConstant.Q1K2_SELECTED_NUM) + 1;
                            option.set(Q1k2SurveyquestionLwConstant.Q1K2_SELECTED_NUM, selectedNum);
                            break;
                        }
                    }
                }
                //小题完成人数
                int finishNum = question.getInt(Q1k2SurveyquestionLwConstant.Q1K2_FINISH_NUM) + 1;
                question.set(Q1k2SurveyquestionLwConstant.Q1K2_FINISH_NUM, finishNum);
                //设置总得分
                BigDecimal totalScore = question.getBigDecimal(Q1k2SurveyquestionLwConstant.Q1K2_TOTAL_SCORE);
                totalScore = totalScore.add(scoreSubmit);
                question.set(Q1k2SurveyquestionLwConstant.Q1K2_TOTAL_SCORE, totalScore);
                BigDecimal avgScore = totalScore.divide(new BigDecimal(finishNum), 2, BigDecimal.ROUND_HALF_UP);
                //设置小题平均分
                question.set(Q1k2SurveyquestionLwConstant.Q1K2_AVG_SCORE, avgScore);
                SaveServiceHelper.save(new DynamicObject[]{question});
            }
            //3.更新调研管理完成情况
            DynamicObject survey = BusinessDataServiceHelper.loadSingle(surveyId, Q1k2SurveyLwConstant.FORMBILLID);
            //完成数量
            int finishNum = survey.getInt(Q1k2SurveyLwConstant.Q1K2_FINISH_NUM) + 1;
            //参与人数
            int joinNum = survey.getInt(Q1k2SurveyLwConstant.Q1K2_JOIN_NUM);
            BigDecimal finishNumDecimal = new BigDecimal(finishNum);
            BigDecimal joinNumDecimal = new BigDecimal(joinNum);
            //完成率
            BigDecimal finishRate = finishNumDecimal.divide(joinNumDecimal, 2, BigDecimal.ROUND_HALF_UP);
            survey.set(Q1k2SurveyLwConstant.Q1K2_FINISH_NUM, finishNum);
            survey.set(Q1k2SurveyLwConstant.Q1K2_FINISH_RATE, finishRate);
            SaveServiceHelper.save(new DynamicObject[]{survey});
            this.getView().invokeOperation("close");
        } else if ("close".equals(e.getEventName())) {
            //退出
            this.getView().invokeOperation("close");
        }
    }
}