package q1k2.pxlx.user.plugin.form;

import com.alibaba.fastjson.JSON;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.form.FormShowParameter;
import kd.bos.form.OpenStyle;
import kd.bos.form.ShowType;
import kd.bos.form.control.Button;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;
import kd.sdk.plugin.Plugin;
import org.jetbrains.annotations.NotNull;
import q1k2.pxlx.train.common.Q1k2SurveyLwConstant;
import q1k2.pxlx.train.common.Q1k2SurveyUserLwConstant;
import q1k2.pxlx.train.common.Q1k2SurveyquestionLwConstant;
import q1k2.pxlx.train.dto.AnswerDto;
import q1k2.pxlx.train.dto.QuestionDto;
import q1k2.pxlx.train.dto.QuestionOptionDto;
import q1k2.pxlx.train.enums.QuestionTypeEnum;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;
import java.util.Map;

/**
 * 调研评估详情动态表单插件
 *
 * @Created 刘维
 */
public class UserSurveyFormPlugin extends AbstractFormPlugin implements Plugin {
    private final String BTN_FINISH = "q1k2_btn_finish";//已完成
    private final String BTN_START = "q1k2_btn_start";//进入
    private final String USER_SURVEY_PAPER = "q1k2_user_survey_paper_lw";//调研评估问卷单据标识

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners(BTN_FINISH, BTN_START);
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        Map<String, Object> customParams = this.getView().getFormShowParameter().getCustomParams();
        this.getModel().setValue("q1k2_picturefield", customParams.get("picture"));
        this.getModel().setValue("q1k2_name", customParams.get("name"));
        this.getModel().setValue("q1k2_user", customParams.get("user"));
        this.getModel().setValue("q1k2_begin_time", customParams.get("beginTime"));
        this.getModel().setValue("q1k2_end_time", customParams.get("endTime"));
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        //状态
        String status = this.getView().getFormShowParameter().getCustomParam("status");
        if ("UNFINISH".equals(status)) {
            //未开始
            //已完成按钮隐藏
            this.getView().setVisible(false, BTN_FINISH);
        } else if ("FINISH".equals(status)) {
            //已完成
            this.getView().setVisible(false, BTN_START);
        }
    }


    @Override
    public void click(EventObject evt) {
        super.click(evt);
        String key = null;
        if (evt.getSource() instanceof Button) {
            key = ((Button) evt.getSource()).getKey();
        }
        if (BTN_START.equals(key) || BTN_FINISH.equals(key)) {
            //进入 或 已完成
            FormShowParameter formShowParameter = this.getView().getFormShowParameter();
            //调研管理id
            Long surveyId = formShowParameter.getCustomParam("surveyId");
            //状态
            String status = formShowParameter.getCustomParam("status");
            //名称
            String name = formShowParameter.getCustomParam("name");
            //查询调研管理
            DynamicObject survey = BusinessDataServiceHelper.loadSingle(surveyId, Q1k2SurveyLwConstant.FORMBILLID);
            //查询调研用户列表
            long currentUserId = UserServiceHelper.getCurrentUserId();
            QFilter surveyUserFilter1 = new QFilter(Q1k2SurveyUserLwConstant.Q1K2_SURVEY, QCP.equals, surveyId);
            QFilter surveyUserFilter2 = new QFilter(Q1k2SurveyUserLwConstant.Q1K2_USER, QCP.equals, currentUserId);
            DynamicObject surveyUser = BusinessDataServiceHelper.loadSingle(Q1k2SurveyUserLwConstant.FORMBILLID, new QFilter[]{surveyUserFilter1, surveyUserFilter2});
            //查询试题数据
            QFilter qFilter = new QFilter(Q1k2SurveyquestionLwConstant.Q1K2_SURVEY, QCP.equals, surveyId);
            String selectFields = "id,q1k2_content,q1k2_type,q1k2_required,q1k2_max_score,q1k2_min_score,q1k2_seq,q1k2_option_content,q1k2_option_score,q1k2_option_seq";
            //DynamicObject[] questions = BusinessDataServiceHelper.load(Q1k2SurveyquestionLwConstant.FORMBILLID, selectFields, qFilter.toArray(), Q1k2SurveyquestionLwConstant.Q1K2_SEQ);
            DynamicObject[] questions = BusinessDataServiceHelper.load(Q1k2SurveyquestionLwConstant.FORMBILLID, selectFields, qFilter.toArray());
            List<QuestionDto> questionDtos = getQuestionDtos(questions, surveyUser);
            FormShowParameter showParameter = new FormShowParameter();
            //跳转调研评估问卷页面
            showParameter.setFormId(USER_SURVEY_PAPER);
            showParameter.setCustomParam("name", name);
            showParameter.setCustomParam("preface", survey.getString(Q1k2SurveyLwConstant.Q1K2_PREFACE));
            showParameter.setCustomParam("questionData", questionDtos);
            showParameter.setCustomParam("status", status);
            showParameter.setCustomParam("surveyId", surveyId);
            OpenStyle openStyle = showParameter.getOpenStyle();
            //显示模式 必填
            openStyle.setShowType(ShowType.Modal);
            this.getView().showForm(showParameter);
        }
    }

    /**
     * 试题对象转试题实体对象
     *
     * @param questions
     * @return
     */
    @NotNull
    private List<QuestionDto> getQuestionDtos(DynamicObject[] questions, DynamicObject surveyUser) {
        List<QuestionDto> questionDtos = new ArrayList<>();
        String answerJson = surveyUser.getString(Q1k2SurveyUserLwConstant.Q1K2_ANSWER_TAG);
        List<AnswerDto> answerDtos = JSON.parseArray(answerJson, AnswerDto.class);
        for (int i = 0; questions != null && i < questions.length; i++) {
            DynamicObject question = questions[i];
            QuestionDto questionDto = new QuestionDto();
            //id
            String questionId = String.valueOf(question.get(Q1k2SurveyquestionLwConstant.ID));
            questionDto.setId(questionId);
            //题目
            questionDto.setContent(question.getString(Q1k2SurveyquestionLwConstant.Q1K2_CONTENT));
            //最高分
            questionDto.setMaxScore(question.getBigDecimal(Q1k2SurveyquestionLwConstant.Q1K2_MAX_SCORE));
            //最低分
            questionDto.setMinScore(question.getBigDecimal(Q1k2SurveyquestionLwConstant.Q1K2_MIN_SCORE));
            //是否必填
            questionDto.setRequired(question.getBoolean(Q1k2SurveyquestionLwConstant.Q1K2_REQUIRED));
            //顺序
            questionDto.setSeq(question.getInt(Q1k2SurveyquestionLwConstant.Q1K2_SEQ));
            //类型
            String type = question.getString(Q1k2SurveyquestionLwConstant.Q1K2_TYPE);
            questionDto.setType(type);
            //类型名称
            String typeName = QuestionTypeEnum.getEnum(type).getName();
            questionDto.setTypeName(typeName);
            //答案
            if (answerDtos != null) {
                for (AnswerDto answer : answerDtos) {
                    if (answer.getQuestionId().equals(questionId)) {
                        questionDto.setAnswer(answer.getAnswer());
                        break;
                    }
                }
            }
            //选项
            DynamicObjectCollection questionOptions = question.getDynamicObjectCollection(Q1k2SurveyquestionLwConstant.ENTRYENTITY);
            List options = new ArrayList();
            for (int j = 0; questionOptions != null && j < questionOptions.size(); j++) {
                DynamicObject questionOption = questionOptions.get(j);
                QuestionOptionDto questionOptionDto = new QuestionOptionDto();
                questionOptionDto.setSeq(questionOption.getInt(Q1k2SurveyquestionLwConstant.Q1K2_OPTION_SEQ));
                questionOptionDto.setId(String.valueOf(questionOption.get("id")));
                questionOptionDto.setScore(questionOption.getBigDecimal(Q1k2SurveyquestionLwConstant.Q1K2_OPTION_SCORE));
                questionOptionDto.setContent(questionOption.getString(Q1k2SurveyquestionLwConstant.Q1K2_OPTION_CONTENT));
                options.add(questionOptionDto);
            }
            questionDto.setOptions(options);
            questionDtos.add(questionDto);
        }
        return questionDtos;
    }
}